import uuid
from abc import abstractmethod, ABC
from dataclasses import dataclass
from typing import Any, Dict

import spacy
from spacy.tokens import Token

from sziszapangma.integration.repository.experiment_repository import ExperimentRepository
from sziszapangma.integration.task.processing_task import ProcessingTask
from sziszapangma.model.relation_manager import RelationManager


@dataclass
class WordSpan:
    text: str
    index_start: int
    index_end: int


class SpacyTokenPosProcessingTask(ProcessingTask, ABC):
    _spacy_model_name: str
    _nlp: Any
    _input_property_name: str
    _spacy_property_name: str

    def __init__(
        self,
        task_name: str,
        spacy_property_name: str,
        require_update: bool,
        input_property_name: str
    ):
        super().__init__(task_name, require_update)
        self._spacy_property_name = spacy_property_name
        self._nlp = spacy.load("pl_core_news_lg")
        self._input_property_name = input_property_name

    def run_single_process(self, record_id: str, experiment_repository: ExperimentRepository,
                           relation_manager: RelationManager):
        test_property = experiment_repository.get_property_for_key(
            record_id, self._input_property_name)
        document = self._nlp(self.get_transcript_to_process(test_property))

        spacy_result = [self.token_to_result_dict(token) for token in document]
        experiment_repository.update_property_for_key(record_id, self._spacy_property_name,
                                                      spacy_result)

    @staticmethod
    def token_to_result_dict(token: Token) -> Dict[str, str]:
        return {'id': str(uuid.uuid4()), 'word': token.text, 'pos': token.pos_}

    @abstractmethod
    def get_transcript_to_process(self, property_value: Any) -> str:
        pass

    def skip_for_record(self, record_id: str, experiment_repository: ExperimentRepository) -> bool:
        return experiment_repository.property_exists(record_id, self._spacy_property_name)
